/* Copyright (C)  NHR@FAU, University Erlangen-Nuremberg.
 * All rights reserved. This file is part of nusif-solver.
 * Use of this source code is governed by a MIT style
 * license that can be found in the LICENSE file.*/
#ifndef __COMM_H_
#define __COMM_H_
#if defined(_MPI)
#include <mpi.h>
#endif

#include "matrix.h"

enum op { MAX = 0, SUM };

typedef struct {
    int rank;
    int size;
#if defined(_MPI)
    MPI_Comm comm;

    int num_external;
    int num_send_neighbors;
    int* external_index;
    int* external_local_index;
    int total_to_be_sent;
    int* elements_to_send;
    int* neighbors;
    int* recv_length;
    int* send_length;
    double* send_buffer;
#endif
} Comm;

extern int sizeOfRank(int rank, int size, int N);
extern void commInit(Comm* c, int argc, char** argv);
extern void commFinalize(Comm* c);
extern void commPartition(Comm*, Matrix* m);
extern void commPrintConfig(Comm*);
extern void commExchange(Comm*, double*);
extern void commReduction(double* v, int op);

static inline int commIsMaster(Comm* c) { return c->rank == 0; }
#endif // __COMM_H_
